@@ -6,6 +6,7 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
                      do.anglex=FALSE,do.angley=FALSE,do.anglez=FALSE,
                      do.roll_med_acc_x=FALSE,do.roll_med_acc_y=FALSE,do.roll_med_acc_z=FALSE,
                      do.dev_roll_med_acc_x=FALSE,do.dev_roll_med_acc_y=FALSE,do.dev_roll_med_acc_z=FALSE,do.enmoa=FALSE,
+                     do.lfen=FALSE,
                      lb = 0.2, hb = 15,  n = 4,meantempcal=c(),chunksize=c(),selectdaysfile=c(),
                      dayborder=0,dynrange=c(),configtz=c(),...) {
   #get input variables
@@ -22,16 +23,15 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
   if (length(which(ls() == "outputdir")) != 0) outputdir = input$outputdir
   if (length(which(ls() == "outputfolder")) != 0) outputfolder = input$outputfolder
   metrics2do = data.frame(do.bfen,do.enmo,do.lfenmo,do.en,do.hfen,
-                    do.hfenplus,do.mad,do.anglex,do.angley,do.anglez,do.roll_med_acc_x,do.roll_med_acc_y,do.roll_med_acc_z,
-                    do.dev_roll_med_acc_x,do.dev_roll_med_acc_y,do.dev_roll_med_acc_z,do.enmoa)
-
+                          do.hfenplus,do.mad,do.anglex,do.angley,do.anglez,do.roll_med_acc_x,do.roll_med_acc_y,do.roll_med_acc_z,
+                          do.dev_roll_med_acc_x,do.dev_roll_med_acc_y,do.dev_roll_med_acc_z,do.enmoa,do.lfen)
   if (length(chunksize) == 0) chunksize = 1
-  if (chunksize > 1) chunksize = 1
+  if (chunksize > 1.5) chunksize = 1.5
   if (chunksize < 0.2) chunksize = 0.2
   nmetrics = sum(c(do.bfen,do.enmo,do.lfenmo,do.en,do.hfen,do.hfenplus,do.mad,
                    do.anglex,do.angley,do.anglez,
                    do.roll_med_acc_x,do.roll_med_acc_y,do.roll_med_acc_z,
-                   do.dev_roll_med_acc_x,do.dev_roll_med_acc_y,do.dev_roll_med_acc_z,do.enmoa))
+                   do.dev_roll_med_acc_x,do.dev_roll_med_acc_y,do.dev_roll_med_acc_z,do.enmoa,do.lfen))
   if (length(nmetrics) == 0) {
     cat("\nWARNING: No metrics selected\n")
   }
@@ -52,20 +52,17 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
     cat(paste("\nshort windowsize has now been automatically adjusted to: ",ws3," seconds in order to meet this criteria.\n",sep=""))
   }
   windowsizes = c(ws3,ws2,ws)
+  data = c()
   PreviousEndPage = c() # needed for g.readaccfile
   start_meas = ws2/60 #ensures that first window starts at logical timepoint relative to its size (15,30,45 or 60 minutes of each hour)
   monnames = c("genea","geneactive","actigraph","axivity") #monitor names
   filequality = data.frame(filetooshort=FALSE,filecorrupt=FALSE,filedoesnotholdday = FALSE,NFilePagesSkipped = 0)
-  # filecorrupt = FALSE
-  # filetooshort = FALSE
-  # filedoesnotholdday = FALSE
   i = 1 #counter to keep track of which binary block is being read
   count = 1 #counter to keep track of the number of seconds that have been read
   count2 = 1 #count number of blocks read with length "ws2" (15 minutes or whatever is specified above)
   LD = 2 #dummy variable used to identify end of file and to make the process stop
   bsc_qc = data.frame(time=c(),size=c(), stringsAsFactors = FALSE)
   # inspect file
-
   if (length(unlist(strsplit(datafile,"[.]RD"))) > 1) {
     useRDA = TRUE
   } else {
@@ -113,9 +110,7 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
   } else if (temp.available == TRUE){
     metalong = matrix(" ",((nev/(sf*ws2))+100),7) #generating output matrix for 15 minutes summaries
   }
-
   #------------------------------------------
-
   if (length(unlist(strsplit(datafile,"[.]RD"))) > 1) {
     useRDA = TRUE
   } else {
@@ -124,6 +119,7 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
   #===============================================
   # Read file
   switchoffLD = 0 #dummy variable part "end of loop mechanism"
+  sforiginal = sf
   while (LD > 1) {
     P = c()
     if (i  == 1) {
@@ -145,6 +141,7 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
       NFilePagesSkipped = filequality$NFilePagesSkipped
       switchoffLD = accread$switchoffLD
       PreviousEndPage = accread$endpage
+      rm(accread); gc()
     } else {
       filetooshort = FALSE
       filecorrupt = FALSE
@@ -174,7 +171,6 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
         } else if (dformat == 4) {
           data = P$data
         }
-
         #add left over data from last time
         if (nrow(S) > 0) {
           data = rbind(S,data)
@@ -195,6 +191,7 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
         if (i == 1 | (i != 1 & length(selectdaysfile) > 0)) { #only do this for first block of data
           starttime = g.getstarttime(datafile=datafile,P=P,header=header,mon=mon,
                                      dformat=dformat,desiredtz=desiredtz,selectdaysfile=selectdaysfile)
+          if (exists("P")) rm(P); gc()
           #==================================================
           #inspection timezone
           timezone = attr(unclass(as.POSIXlt(starttime[1])),which="tzone")
@@ -245,11 +242,6 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
           data = data[-c(1:floor(sampleshift)),] #delete data accordingly
           newmin = start_min+minshift #recalculate first timestamp
           newsec = 0
-          # newsec = start_sec+secshift # Removed as suggested by E Mirkes
-          # if (newsec >= 60) { # Removed as suggested by E Mirkes
-          #   newsec = newsec - 60 # Removed as suggested by E Mirkes
-          #   newmin = newmin + 1 # Removed as suggested by E Mirkes
-          # } # Removed as suggested by E Mirkes
           remem2add24 = FALSE
           if (newmin >= 60) {
             newmin = newmin - 60
@@ -291,16 +283,16 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
             starttime = as.POSIXlt(as.numeric(starttime) + (24*3600),origin="1970-01-01")
           }
         }
+        if (i != 0 & length(selectdaysfile) == 0 & exists("P")) rm(P); gc()
         LD = nrow(data)
         if (LD < (ws*sf) & i == 1) {
-          cat("\nError: data too short for doing non-wear detection 3\n")
+          warning('\nWarning data too short for doing non-wear detection 3\n')
           switchoffLD = 1
           LD = 0 #ignore rest of the data and store what has been loaded so far.
         }
       } else { # if useRDA == TRUE
-        LD = length(Gx)
+        LD = nrow(data)
       }
-      # if (useRDA == TRUE) LD = 1
       #store data that could not be used for this block, but will be added to next block
       if (LD >= (ws*sf)) { # LD != 0
         if (useRDA == FALSE) {
@@ -314,49 +306,12 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
             S = matrix(0,0,ncol(data))
           }
           data = as.matrix(data[1:use,])
-
           LD = nrow(data) #redefine LD because there is less data
           ##==================================================
           # Feature calculation
           dur = nrow(data)	#duration of experiment in data points
           durexp = nrow(data) / (sf*ws)	#duration of experiment in hrs
           data = as.matrix(data)
-          # Initialization of variables
-          if (mon == 1) {
-            data[,1:3] = scale(data[,1:3],center = -offset, scale = 1/scale) #rescale data
-            Gx = as.numeric(data[,1]); Gy = as.numeric(data[,2]); Gz = as.numeric(data[,3])
-          } else if (mon == 4 & dformat == 3) {
-            data[,1:3] = scale(data[,1:3],center = -offset, scale = 1/scale) #rescale data
-            Gx = as.numeric(data[,1]); Gy = as.numeric(data[,2]); Gz = as.numeric(data[,3])
-          } else if (mon == 4 & (dformat == 4 |  dformat == 2)) {
-            data[,2:4] = scale(data[,2:4],center = -offset, scale = 1/scale) #rescale data
-            Gx = as.numeric(data[,2]); Gy = as.numeric(data[,3]); Gz = as.numeric(data[,4])
-          } else if (mon == 2 & dformat == 1) {
-            yy = as.matrix(cbind(as.numeric(data[,7]),as.numeric(data[,7]),as.numeric(data[,7])))
-            data[,2:4] = scale(as.matrix(data[,2:4]),center = -offset, scale = 1/scale) +
-              scale(yy, center = rep(meantemp,3), scale = 1/tempoffset)  #rescale data
-            Gx = as.numeric(data[,2]); Gy = as.numeric(data[,3]); Gz = as.numeric(data[,4])
-          } else if (dformat == 2 & (mon != 4)) {
-            data2 = matrix(NA,nrow(data),3)
-            if (ncol(data) == 3) extra = 0
-            if (ncol(data) >= 4) extra = 1
-            for (jij in 1:3) {
-              data2[,jij] = as.numeric(data[,(jij+extra)])
-            }
-            if (mon == 3) {
-              data2[,1:3] = scale(data2[,1:3],center = -offset, scale = 1/scale)  #rescale data
-            } else if (mon == 2) {
-              yy = as.matrix(cbind(as.numeric(data[,7]),as.numeric(data[,7]),as.numeric(data[,7])))
-              meantemp = mean(as.numeric(data[,7]))
-              # meantemp replaced by meantempcal # 19-12-2013
-              if (length(meantempcal) == 0) {
-                meantempcal = meantemp
-              }
-              data2[,1:3] = scale(data2[,1:3],center = -offset, scale = 1/scale) +
-                scale(yy, center = rep(meantempcal,3), scale = 1/tempoffset)  #rescale data
-            }
-            Gx = as.numeric(data2[,1]); Gy = as.numeric(data2[,2]); Gz = as.numeric(data2[,3])
-          }
           #--------------------------------------------
           if (mon == 2 | (mon == 4 & dformat == 4)) {
             if (mon == 2) {
@@ -367,9 +322,66 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
             light = as.numeric(data[,lightcolumn])
             temperature = as.numeric(data[,temperaturecolumn])
           }
+          # Initialization of variables
+          if (mon == 1) {
+            data = data[,1:3]
+            data[,1:3] = scale(data[,1:3],center = -offset, scale = 1/scale) #rescale data
+          } else if (mon == 4 & dformat == 3) {
+            data = data[,1:3]
+            data[,1:3] = scale(data[,1:3],center = -offset, scale = 1/scale) #rescale data
+          } else if (mon == 4 & (dformat == 4 |  dformat == 2)) {
+            data = data[,2:4]
+            data[,1:3] = scale(data[,1:3],center = -offset, scale = 1/scale) #rescale data
+          } else if (mon == 2 & dformat == 1) {
+            yy = as.matrix(cbind(as.numeric(data[,7]),as.numeric(data[,7]),as.numeric(data[,7])))
+            data = data[,2:4]
+            data[,1:3] = scale(as.matrix(data[,1:3]),center = -offset, scale = 1/scale) +
+              scale(yy, center = rep(meantemp,3), scale = 1/tempoffset)  #rescale data
+            rm(yy); gc()
+          } else if (dformat == 2 & (mon != 4)) {
+            if (mon == 2) {
+              yy = as.matrix(cbind(as.numeric(data[,7]),as.numeric(data[,7]),as.numeric(data[,7])))
+              meantemp = mean(as.numeric(data[,7]))
+              if (length(meantempcal) == 0) meantempcal = meantemp
+            }
+            if (ncol(data) == 3) data = data[,1:3]
+            if (ncol(data) >= 4) data = data[,2:4]
+            if (mon == 3) {
+              data[,1:3] = scale(data[,1:3],center = -offset, scale = 1/scale)  #rescale data
+            } else if (mon == 2) {
+              # meantemp replaced by meantempcal # 19-12-2013
+              data[,1:3] = scale(data[,1:3],center = -offset, scale = 1/scale) +
+                scale(yy, center = rep(meantempcal,3), scale = 1/tempoffset)  #rescale data
+              rm(yy); gc()
+            }
+          }
+          suppressWarnings(storage.mode(data) <- "numeric")
+          ## resample experiment to see whehter processing time can be much improved if data is resampled
+          sfold = sforiginal # keep sf, because light, temperature are not resampled at the moment
+          # if (sforiginal > 30) {
+          #   rawTime = seq(0,nrow(data),by=1) / sforiginal
+          #   sfnew = 30
+          #   step = 1/sfnew
+          #   start = rawTime[1]
+          #   end = rawTime[length(rawTime)]
+          #   timeRes = seq(start, end, step)
+          #   nr = length(timeRes) - 1
+          #   timeRes = as.vector(timeRes[1:nr])
+          #   rawLast=nrow(data)
+          #   # print("==========================")
+          #   # print(paste0("nrow before: ", nrow(data)))
+          #   data = GGIR::resample(data, rawTime, timeRes, rawLast)
+          #   # print(paste0("nrow before: ", nrow(data)))
+          #   sf = sfnew
+          #   # print(paste0("new sf: ",sfnew))
+          #   # print("==========================")
+          #   LD = nrow(data)
+          #   dur = nrow(data)	#duration of experiment in data points
+          #   durexp = nrow(data) / (sf*ws)	#duration of experiment in hrs
+          # }
           #####
           # STORE THE RAW DATA
-          # Gx, Gy, Gz, starttime, (temperature, light)
+          # data[,1], data[,2], data[,3], starttime, (temperature, light)
           if (length(selectdaysfile) > 0) {
             path3 = paste(outputdir,outputfolder,sep="") #where is output stored?
             # calculate extra timestamp in a more complete format
@@ -378,25 +390,20 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
             # starttime_aschar_tz = strftime(as.POSIXlt(as.POSIXct(starttime),tz=desiredtz),format="%Y-%m-%d %H:%M:%S %z")
             if (mon == 2 | (mon == 4 & dformat == 4)) {
               I = INFI
-              save(I,sf,wday,wdayname,decn,Gx,Gy,Gz,starttime,temperature,light,
+              save(I,sf,wday,wdayname,decn,data,starttime,temperature,light,
                    file = paste(path3,"/meta/raw/",filename,"_day",i,".RData",sep=""))
             } else {
-              save(I,sf,wday,wdayname,decn,Gx,Gy,Gz,starttime,
+              save(I,sf,wday,wdayname,decn,data,starttime,
                    file = paste(path3,"/meta/raw/",filename,"_day",i,".RData",sep=""))
             }
           }
         } else {
           #           load(datafile) # turned off because datafile will already be loaded (earlier on in script)
-          data = cbind(rep(0,length(Gx)),Gx,Gy,Gz)
+          data = cbind(rep(0,nrow(data)),data)
           LD = nrow(data)
         }
-        EN = sqrt(Gx^2 + Gy^2 + Gz^2)
-        allmetrics = g.applymetrics(Gx,Gy,Gz,n,sf,ws3,metrics2do)
-        # attach(allmetrics,warn.conflicts = FALSE)
-        # globalVariables(c("BFEN3b","ENMO3b","LFENMO3b","EN3b","HFEN3b",
-        #                   "HFENplus3b", "MAD3b", "angle_x3b","angle_y3b","angle_z3b",
-        #                   "roll_med_acc_x3b","roll_med_acc_y3b","roll_med_acc_z3b","dev_roll_med_acc_x3b",
-        #                   "dev_roll_med_acc_y3b","dev_roll_med_acc_z3b"))
+        EN = sqrt(data[,1]^2 + data[,2]^2 + data[,3]^2)
+        allmetrics = g.applymetrics(data = data,n=n,sf=sf,ws3=ws3,metrics2do=metrics2do, lb=lb,hb=hb)
         BFEN3b = allmetrics$BFEN3b
         ENMO3b = allmetrics$ENMO3b
         ENMOa3b = allmetrics$ENMOa3b
@@ -414,6 +421,7 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
         dev_roll_med_acc_x3b = allmetrics$dev_roll_med_acc_x3b
         dev_roll_med_acc_y3b = allmetrics$dev_roll_med_acc_y3b
         dev_roll_med_acc_z3b = allmetrics$dev_roll_med_acc_z3b
+        LFEN3b = allmetrics$LFEN3b
       }
       if (LD >= (ws*sf)) { #LD != 0
         #-----------------------------------------------------
@@ -423,7 +431,7 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
           metashort = rbind(metashort,extension)
           extension2 = matrix(" ",((3600/ws2) *24),ncol(metalong)) #add another day to metashort once you reach the end of it
           metalong = rbind(metalong,extension2)
-
+          
           cat("\nvariable metashort extended\n")
         }
         col_msi = 2
@@ -478,9 +486,11 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
         if (do.enmoa == TRUE) {
           metashort[count:(count-1+length(ENMOa3b)),col_msi] = ENMOa3b; col_msi = col_msi + 1
         }
+        if (do.lfen == TRUE) {
+          metashort[count:(count-1+length(LFEN3b)),col_msi] = LFEN3b; col_msi = col_msi + 1
+        }
         count = count + length(EN3b) #increasing "count" the indicator of how many seconds have been read
-
-        rm(Gx); rm(Gy); rm(Gz); rm(allmetrics)
+        rm(allmetrics)
         # update blocksize depending on available memory
         BlocksizeNew = updateBlocksize(blocksize=blocksize, bsc_qc=bsc_qc)
         bsc_qc = BlocksizeNew$bsc_qc
@@ -510,21 +520,9 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
               hoc1=(((h-1)*window2)+ window2*0.5 ) - window*0.5
               hoc2=(((h-1)*window2)+ window2*0.5 ) + window*0.5
             }
-            if (dformat == 1) {
-              sdwacc = sd(as.numeric(data[(1+hoc1):hoc2,(jj+(mon-1))]),na.rm=TRUE)
-              maxwacc = max(as.numeric(data[(1+hoc1):hoc2,(jj+(mon-1))]),na.rm=TRUE)
-              minwacc = min(as.numeric(data[(1+hoc1):hoc2,(jj+(mon-1))]),na.rm=TRUE)
-            } else if (dformat == 2 | dformat == 4) {
-              if (ncol(data) == 3) extra = 0
-              if (ncol(data) >= 4) extra = 1
-              sdwacc = sd(as.numeric(data[(1+hoc1):hoc2,(jj+extra)]),na.rm=TRUE)
-              maxwacc = max(as.numeric(data[(1+hoc1):hoc2,(jj+extra)]),na.rm=TRUE)
-              minwacc = min(as.numeric(data[(1+hoc1):hoc2,(jj+extra)]),na.rm=TRUE)
-            } else if (dformat == 3) {
-              sdwacc = sd(as.numeric(data[(1+hoc1):hoc2,jj]),na.rm=TRUE)
-              maxwacc = max(as.numeric(data[(1+hoc1):hoc2,jj]),na.rm=TRUE)
-              minwacc = min(as.numeric(data[(1+hoc1):hoc2,jj]),na.rm=TRUE)
-            }
+            sdwacc = sd(as.numeric(data[(1+hoc1):hoc2,jj]),na.rm=TRUE)
+            maxwacc = max(as.numeric(data[(1+hoc1):hoc2,jj]),na.rm=TRUE)
+            minwacc = min(as.numeric(data[(1+hoc1):hoc2,jj]),na.rm=TRUE)
             #estimate number of data points of clipping based on raw data at about 87 Hz
             if (length(dynrange) > 0) {
               clipthres = dynrange - 0.5
@@ -539,16 +537,7 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
                 clipthres = 7.5 # hard coded assumption that dynamic range is 8g
               }
             }
-
-            if (dformat == 1) {
-              CW[h,jj] = length(which(abs(as.numeric(data[(1+cliphoc1):cliphoc2,(jj+(mon-1))])) > clipthres))
-            } else if (dformat == 2 | dformat == 4) {
-              if (ncol(data) == 3) extra = 0
-              if (ncol(data) >= 4) extra = 1
-              CW[h,jj] = length(which(abs(as.numeric(data[(1+cliphoc1):cliphoc2,(jj+extra)])) > clipthres))
-            } else if (dformat == 3) {
-              CW[h,jj] = length(which(abs(as.numeric(data[(1+cliphoc1):cliphoc2,jj])) > clipthres))
-            }
+            CW[h,jj] = length(which(abs(as.numeric(data[(1+cliphoc1):cliphoc2,jj])) > clipthres))
             #non-wear criteria are monitor specific
             if (mon == 1) {
               sdcriter = 0.003
@@ -569,7 +558,6 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
               }
             } else {
             }
-
           }
           CW = CW / (window2) #changed 30-1-2012, was window*sf
           NWav[h,1] = (NW[h,1] + NW[h,2] + NW[h,3]) #indicator of non-wear
@@ -581,28 +569,30 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
         if (mon == 2 | (mon == 4 & dformat == 4)) { #going from sample to ws2
           #light (running mean)
           lightc = cumsum(c(0,light))
-          select = seq(1,length(lightc),by=(ws2*sf))
+          select = seq(1,length(lightc),by=(ws2*sfold))
           lightmean = diff(lightc[round(select)]) / abs(diff(round(select)))
+          rm(lightc); gc()
           #light (running max)
           lightmax = matrix(0,length(lightmean),1)
-          for (li in 1:(length(light)/(ws2*sf))) {
-            tempm = max(light[((li-1)*(ws2*sf)):(li*(ws2*sf))])
+          for (li in 1:(length(light)/(ws2*sfold))) {
+            tempm = max(light[((li-1)*(ws2*sfold)):(li*(ws2*sfold))])
             if (length(tempm) > 0) {
               lightmax[li] = tempm[1]
             } else {
-              lightmax[li] = max(light[((li-1)*(ws2*sf)):(li*(ws2*sf))])
+              lightmax[li] = max(light[((li-1)*(ws2*sfold)):(li*(ws2*sfold))])
             }
           }
           #temperature (running mean)
           temperaturec = cumsum(c(0,temperature))
-          select = seq(1,length(temperaturec),by=(ws2*sf))
+          select = seq(1,length(temperaturec),by=(ws2*sfold))
           temperatureb = diff(temperaturec[round(select)]) / abs(diff(round(select)))
+          rm(temperaturec); gc()
         }
         #EN going from sample to ws2
-
         ENc = cumsum(c(0,EN))
-        select = seq(1,length(ENc),by=(ws2*sf))
+        select = seq(1,length(ENc),by=(ws2*sf)) #<= EN is derived from data, so it needs the new sf
         ENb = diff(ENc[round(select)]) / abs(diff(round(select)))
+        rm(ENc, EN); gc()
         if (mon == 2 | (mon == 4 & dformat == 4)) {
           metalong[(count2):((count2-1)+nrow(NWav)),col_mli] = lightmean; col_mli= col_mli + 1
           metalong[(count2):((count2-1)+nrow(NWav)),col_mli] = lightmax; col_mli= col_mli + 1
@@ -610,16 +600,16 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
         }
         metalong[(count2):((count2-1)+nrow(NWav)),col_mli] = ENb; col_mli= col_mli + 1
         count2  = count2 + nmin
-        rm(P)
+        if (exists("data")) rm(data)
+        if (exists("light")) rm(light)
+        if (exists("temperature")) rm(temperature)
         gc()
       } #end of section which is skipped when switchoff == 1
     } else {
       LD = 0 #once LD < 1 the analysis stops, so this is a trick to stop it
-      # cat("\nstop reading because there is not enough data in this block\n")
-    }
-    if (switchoffLD == 1) {
-      LD = 0
+      # stop reading because there is not enough data in this block
     }
+    if (switchoffLD == 1) LD = 0
     if (ceiling(daylimit) != FALSE) {
       if (i == ceiling(daylimit)) { #to speed up testing only read first 'i' blocks of data
         LD = 0 #once LD < 1 the analysis stops, so this is a trick to stop it
@@ -644,11 +634,10 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
         SDF = read.csv(selectdaysfile)
         if (useRDA == FALSE) I = g.inspectfile(datafile, desiredtz=desiredtz)
         hvars = g.extractheadervars(I)
-        SN = hvars$SN
-        SDFi = which(as.numeric(SDF$Monitor) == as.numeric(SN))
+        deviceSerialNumber = hvars$deviceSerialNumber
+        SDFi = which(as.numeric(SDF$Monitor) == as.numeric(deviceSerialNumber))
         dateday1 = as.character(SDF[SDFi,2])
         dateday2 = as.character(SDF[SDFi,3])
-
         dtday1 = as.POSIXlt(paste0(dateday1," 01:00:00"),format="%d/%m/%Y %H:%M:%S")
         dtday2 = as.POSIXlt(paste0(dateday2," 01:00:00"),format="%d/%m/%Y %H:%M:%S")
         deltat = as.numeric(dtday2) - as.numeric(dtday1)
@@ -676,16 +665,16 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
     if (nrow(metalong) > 2) {
       starttime4 = round(as.numeric(starttime)) #numeric time but relative to the desiredtz
       time1 = seq(starttime4,(starttime4+(nrow(metalong)*ws2)-1),by=ws2)
-
+      
       if (length(selectdaysfile) > 0 & round((24*(3600/ws2))+1) < length(time1)) { # (Millenium cohort)
         #===================================================================
         # All of the below needed for Millenium cohort
         SDF = read.csv(selectdaysfile)
         if (useRDA == FALSE) I = g.inspectfile(datafile, desiredtz=desiredtz)
         hvars = g.extractheadervars(I)
-        SN = hvars$SN
-        SDFi = which(as.numeric(SDF$Monitor) == as.numeric(SN))
-        if (length(SDFi) == 1) { # if SN is not in the file then this is skipped
+        deviceSerialNumber = hvars$deviceSerialNumber
+        SDFi = which(as.numeric(SDF$Monitor) == as.numeric(deviceSerialNumber))
+        if (length(SDFi) == 1) { # if deviceSerialNumber is not in the file then this is skipped
           dateday1 = as.character(SDF[SDFi,2])
           dateday2 = as.character(SDF[SDFi,3])
           dtday1 = as.POSIXlt(paste0(dateday1," 01:00:00"),format="%d/%m/%Y %H:%M:%S")
@@ -706,18 +695,17 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
     }
     metricnames_short = c("timestamp","BFEN","ENMO","LFENMO","EN","HFEN","HFENplus","MAD",
                           "anglex","angley","anglez","roll_med_acc_x","roll_med_acc_y","roll_med_acc_z",
-                          "dev_roll_med_acc_x","dev_roll_med_acc_y","dev_roll_med_acc_z","ENMOa") #
+                          "dev_roll_med_acc_x","dev_roll_med_acc_y","dev_roll_med_acc_z","ENMOa","LFEN") #
     metricnames_short = as.character(metricnames_short[c(TRUE,do.bfen,do.enmo,do.lfenmo,do.en,do.hfen,do.hfenplus,do.mad,
                                                          do.anglex,do.angley,do.anglez,
                                                          do.roll_med_acc_x,do.roll_med_acc_y,do.roll_med_acc_z,
                                                          do.dev_roll_med_acc_x,do.dev_roll_med_acc_y,do.dev_roll_med_acc_z,
-                                                         do.enmoa)]) #
+                                                         do.enmoa,do.lfen)])
     metashort = data.frame(A = metashort,stringsAsFactors = FALSE)
     names(metashort) = metricnames_short
     for (ncolms in 2:ncol(metashort)) {
       metashort[,ncolms] = as.numeric(metashort[,ncolms])
     }
-
     if (mon == 1 | mon == 3 | (mon == 4 & dformat == 3) | (mon == 4 & dformat == 2)) {
       metricnames_long = c("timestamp","nonwearscore","clippingscore","en")
     } else if (mon == 2 | (mon == 4 & dformat == 4)) {
@@ -728,13 +716,11 @@ g.getmeta = function(datafile,desiredtz = c(),windowsizes = c(5,900,3600),
     for (ncolml in 2:ncol(metalong)) {
       metalong[,ncolml] = as.numeric(metalong[,ncolml])
     }
-
-    closeAllConnections()
+    
+    # closeAllConnections()
   } else {
     metalong=metashort=wday=wdayname=windowsizes = c()
   }
-
-
   # detach(allmetrics,warn.conflicts = FALSE)
   if (length(metashort) == 0 | filedoesnotholdday == TRUE) filetooshort = TRUE
   invisible(list(filecorrupt=filecorrupt,filetooshort=filetooshort,NFilePagesSkipped=NFilePagesSkipped,
