@@ -84,16 +84,12 @@ g.part4 = function(datadir=c(),metadatadir=c(),f0=f0,f1=f1,idloc=1,loglocation =
   if (storefolderstructure == TRUE) {
     filelist = FALSE
     if (length(datadir) == 1) { #could be a directory or one file
-      if (length(unlist(strsplit(datadir,"[.]bi")))>1) filelist = TRUE
-      if (length(unlist(strsplit(datadir,"[.]cs")))>1) filelist = TRUE
-      if (length(unlist(strsplit(datadir,"[.]cw")))>1) filelist = TRUE
+      if (length(unlist(strsplit(datadir,split = "[.](cs|bi|cw)")))>1) filelist = TRUE
     } else { #multiple files
       filelist = TRUE    
     }
     if (filelist == FALSE) {
-      fnamesfull = c(dir(datadir,recursive=TRUE,pattern="[.]csv"),
-                     dir(datadir,recursive=TRUE,pattern="[.]bin"),
-                     dir(datadir,recursive=TRUE,pattern="[.]cwa"))
+      fnamesfull = dir(datadir,recursive=TRUE,pattern="[.](csv|bin|cwa)")
     } else {
       fnamesfull = datadir
     }
@@ -121,21 +117,16 @@ g.part4 = function(datadir=c(),metadatadir=c(),f0=f0,f1=f1,idloc=1,loglocation =
   convertHRsinceprevMN2Clocktime = function(x) {
     # x = hours Since Previous Midnight
     HR = floor(x)
-    MI = floor((x - floor(x)) * 60)
+    MI = floor((x - HR) * 60)
     SE = round(((x - HR) - (MI/60)) * 3600)
-    if (SE == 60) {
-      MI = MI + 1; SE = 0
-    }
-    if (MI == 60) {
-      HR = HR + 1; MI = 0
-    }
+    if (SE == 60) MI = MI + 1; SE = 0
+    if (MI == 60) HR = HR + 1; MI = 0
     if (HR == 24) HR = 0
     if (HR < 10) HR = paste0("0",HR)
     if (MI < 10) MI = paste0("0",MI)
     if (SE < 10) SE = paste0("0",SE)
     return(paste0(HR,":",MI,":",SE))
   }
-  
   #=================================================================
   #=================================================================
   # start of loop through the participants
@@ -144,14 +135,9 @@ g.part4 = function(datadir=c(),metadatadir=c(),f0=f0,f1=f1,idloc=1,loglocation =
     if (overwrite == TRUE) {
       skip = 0 # this will make that analyses is done regardless of whether it was done before
     } else {
+      skip = 0 #do not skip this file
       if (length(ffdone) > 0) {
-        if (length(which(ffdone == fnames[i])) > 0) { 
-          skip = 1 #skip this file because it was analysed before")
-        } else {
-          skip = 0 #do not skip this file
-        }
-      } else {
-        skip = 0
+        if (length(which(ffdone == fnames[i])) > 0) skip = 1 #skip this file because it was analysed before")
       }
     }
     if (skip == 0) {
@@ -175,7 +161,7 @@ g.part4 = function(datadir=c(),metadatadir=c(),f0=f0,f1=f1,idloc=1,loglocation =
       }
       colnames(nightsummary) = colnamesnightsummary
       sumi = 1 # counter to keep track of where we are in filling the output matrix 'nightsummary'
-      lightson = lightsout = L5list = sib.cla.sum = c()
+      sptwindow_HDCZA_end = sptwindow_HDCZA_start = L5list = sib.cla.sum = c()
       # load milestone 3 data (RData files), check whether there is data, identify id numbers...
       load(paste(meta.sleep.folder,"/",fnames[i],sep=""))
       if (nrow(sib.cla.sum) != 0) { #there needs to be some information
@@ -202,7 +188,7 @@ g.part4 = function(datadir=c(),metadatadir=c(),f0=f0,f1=f1,idloc=1,loglocation =
               accid[h] = as.character(unlist(strsplit(accid[h],letter[h]))[1])
             }
           } 
-          accid = as.numeric(accid)
+          accid = suppressWarnings(as.numeric(accid))
           #catch for files with only id in filename and for whom the above attempt to extract the id failed:
           if (is.na(accid) == TRUE) accid = accid_bu
         } else { # get id from filename
@@ -214,28 +200,33 @@ g.part4 = function(datadir=c(),metadatadir=c(),f0=f0,f1=f1,idloc=1,loglocation =
         }
         # get matching identifier from sleeplog
         if (dolog == TRUE) {
+          accid_num = suppressWarnings(as.numeric(accid))
           if (sleeplogidnum == FALSE) {
             wi = which(as.character(sleeplog$id) == as.character(accid))
             if (length(wi) == 0) {
-              wi_alternative = which(sleeplog$id == as.numeric(accid))
+              wi_alternative = which(sleeplog$id == accid_num)
               if (length(wi_alternative) > 0) {
-                cat("\nWarning: argument sleeplogidnum is set to FALSE, but it seems the identifiers are
+                warning("\nArgument sleeplogidnum is set to FALSE, but it seems the identifiers are
                     stored as numeric values, you may want to consider changing sleeplogidnum to TRUE")
               } else {
-                cat(paste0("\nWarning: sleeplog id is stored as format: ", as.character(sleeplog$id[1]),", while
+                warning(paste0("\nSleeplog id is stored as format: ", as.character(sleeplog$id[1]),", while
                            code expects format: ",as.character(accid[1])))
               }
             }
           } else {
-            wi = which(sleeplog$id == as.numeric(accid))
+            
+            wi = which(sleeplog$id == accid_num)
             if (length(wi) == 0) {
               wi_alternative = which(as.character(sleeplog$id) == as.character(accid))
               if (length(wi_alternative) > 0) {
-                cat("\nWarning: argument sleeplogidnum is set to TRUE, but it seems the identifiers are
-                    stored as characrter values, you may want to consider changing sleeplogidnum to TRUE")
+                warning("\nArgument sleeplogidnum is set to TRUE, but it seems the identifiers are
+                    stored as character values, you may want to consider changing sleeplogidnum to TRUE")
               } else {
-                cat(paste0("\nWarning: sleeplog id is stored as format: ", as.character(sleeplog$id[1]),", while
+                
+                if (is.na(accid_num) == TRUE) { # format probably incorrect
+                  warning(paste0("\nSleeplog id is stored as format: ", as.character(sleeplog$id[1]),", while
                            code expects format: ",as.character(accid[1])))
+                }
               }
             }
           }
@@ -275,16 +266,16 @@ g.part4 = function(datadir=c(),metadatadir=c(),f0=f0,f1=f1,idloc=1,loglocation =
           # get default onset and wake (based on sleeplog or on heuristic algorithms)
           # def.noc.sleep is an input argument the GGIR user can use
           # to specify what detection strategy is used in the absense of a sleep diary
-          if (length(def.noc.sleep) == 0 | length(lightsout) == 0) {
+          if (length(def.noc.sleep) == 0 | length(sptwindow_HDCZA_start) == 0) {
             # use L5+/-6hr algorithm if HDCZA fails OR if the user explicitely asks for it (length zero argument)
             if (length(L5list) > 0) {
               defaultSptOnset = L5list[j] - 6
               defaultSptWake = L5list[j] + 6
             }
-          } else if (length(def.noc.sleep) == 1 | length(loglocation) != 0 & length(lightsout) != 0) { 
+          } else if (length(def.noc.sleep) == 1 | length(loglocation) != 0 & length(sptwindow_HDCZA_start) != 0) { 
             # use HDCZA algorithm (inside the g.sib.det function) as backup for sleeplog OR if user explicitely asks for it
-            defaultSptOnset = lightsout[j]
-            defaultSptWake = lightson[j]
+            defaultSptOnset = sptwindow_HDCZA_start[j]
+            defaultSptWake = sptwindow_HDCZA_end[j]
           } else if (length(def.noc.sleep) == 2) {
             # use constant onset and waking time as specified with def.noc.sleep argument
             defaultSptOnset = def.noc.sleep[1] #onset
@@ -303,11 +294,9 @@ g.part4 = function(datadir=c(),metadatadir=c(),f0=f0,f1=f1,idloc=1,loglocation =
             #-----------------------------------------------------------
             #If sleep log is not available available, use default values calculated above (with the heuristic algorithm HDCZA or if that fails L5+/-6hr.
             if (j == nnights.list[1]) sleeplog.t = data.frame(matrix(0,length(nnightlist),5))
-            sleeplog.t[nightj,1] = accid
-            sleeplog.t[nightj,2] = j
-            sleeplog.t[nightj,3] = defaultdur
-            sleeplog.t[nightj,4] = convertHRsinceprevMN2Clocktime(defaultSptOnset)
-            sleeplog.t[nightj,5] = convertHRsinceprevMN2Clocktime(defaultSptWake)
+            sleeplog.t[nightj,1:5] = c(accid, j, defaultdur, 
+                                       convertHRsinceprevMN2Clocktime(defaultSptOnset),
+                                       convertHRsinceprevMN2Clocktime(defaultSptWake))
             names(sleeplog.t) = c("id","night","duration","sleeponset","sleepwake")
             sleeplog_used[i] = FALSE
             cleaningcode = 1
@@ -725,12 +714,8 @@ g.part4 = function(datadir=c(),metadatadir=c(),f0=f0,f1=f1,idloc=1,loglocation =
                     HR = floor(x)
                     MI = floor((x - floor(x)) * 60)
                     SE = round(((x - HR) - (MI/60)) * 3600)
-                    if (SE == 60) {
-                      MI = MI + 1; SE = 0
-                    }
-                    if (MI == 60) {
-                      HR = HR + 1; MI = 0
-                    }
+                    if (SE == 60) MI = MI + 1; SE = 0
+                    if (MI == 60) HR = HR + 1; MI = 0
                     if (HR == 24) HR = 0
                     if (HR < 10) HR = paste0("0",HR)
                     if (MI < 10) MI = paste0("0",MI)
@@ -839,16 +824,13 @@ g.part4 = function(datadir=c(),metadatadir=c(),f0=f0,f1=f1,idloc=1,loglocation =
           ##########################################################################
         } #nights
         if (length(nnights.list) == 0) { #if there were no nights to analyse
-          nightsummary[sumi,1] = accid
-          nightsummary[sumi,2] = 0 #night
-          nightsummary[sumi,3:25] = NA #night
+          nightsummary[sumi,1:2] = c(accid, 0)
+          nightsummary[sumi,3:25] = NA
           nightsummary[sumi,26] = fnames[i]
           nightsummary[sumi,27] = 4 #cleaningcode = 4 (no nights of accelerometer available)
-          nightsummary[sumi,28] = FALSE #sleeplog_used[i]
-          nightsummary[sumi,29] = TRUE #acc_available
+          nightsummary[sumi,28:29] = c(FALSE, TRUE) #sleeplog_used acc_available
           if (storefolderstructure == TRUE) {
-            nightsummary[sumi,30] = ffd[i] #full filename structure
-            nightsummary[sumi,31] = ffp[i] #use the lowest foldername as foldername name
+            nightsummary[sumi,30:31] = c(ffd[i], ffp[i]) #full filename structure and use the lowest foldername as foldername name
           }
           sumi = sumi + 1
         }
